/////////////////////////////////////////////////////////////////////////////////////////
//
// DS18B20 Library V2
//
// --------------------------------------------------------------------------------------
// 
// Filename:      ds18b20.c
// Version:       2.0
// Date:          01/11/2011
// Author:        Joel Guittet - http://myfreescalewebpage.free.fr
//
/////////////////////////////////////////////////////////////////////////////////////////
//
// Description
//
// This files contains the functions used to communicate with the DS18B20 temperature
// sensor throw the 1-Wire interface.
//
/////////////////////////////////////////////////////////////////////////////////////////
//
// Revisions
//
// Version	| Author		| Description
// --------------------------------------------------------------------------------------
//   		|               |
//
/////////////////////////////////////////////////////////////////////////////////////////


//---------------------------------------------------------------------------------------
// Includes
//---------------------------------------------------------------------------------------

#include "ds18b20.h"


//---------------------------------------------------------------------------------------
// Name:        DS18B20_Init
// Param:		Device: pointer on the DS18B20 device structure
//				PortData: Port Data register on which the sensor is connected (1-Wire Interface)
//				PortDataDirection: Port Data Direction register on which the sensor is connected (1-Wire Interface)
//				Bit: bit on which the sensor is connected (1-Wire Interface)
//				Resolution: DS18B20 resolution (9, 10, 11 or 12 bits)
// Return:      1-Wire Reset result: NO_DEVICE_DETECTED, DEVICE_PRESENT or DEVICE_SHORTED
//
// Description:	Initialization of the DS18B20 device
//---------------------------------------------------------------------------------------
t_OneWireResetResult DS18B20_Init(t_DS18B20Device * pxDevice, UINT8 * pu8PortData, UINT8 * pu8PortDataDirection, UINT8 u8Bit, UINT8 u8Resolution)
{
	/* First initialize the DS18B20 Device structure */
	pxDevice->OneWireInterface.PortData = pu8PortData;
	pxDevice->OneWireInterface.PortDataDirection = pu8PortDataDirection;
	pxDevice->OneWireInterface.Bit = u8Bit;
	pxDevice->Status = NO_DEVICE_DETECTED;
	pxDevice->Temperature = 0;
	
	/* Initialize the 1-Wire interface */
	ONEWIRE_Init(&pxDevice->OneWireInterface);
	
	/* Check the resolution */
	if (u8Resolution < 9 || u8Resolution > 12) return NO_DEVICE_DETECTED;
	
	/* Convert the resolution */
	u8Resolution = 0x1F + ((u8Resolution - 9) << 5);
	
	/* To write in the scratchpad, perform a 1-wire reset */
	pxDevice->Status = ONEWIRE_Reset(&pxDevice->OneWireInterface);
	
	if (pxDevice->Status == DEVICE_PRESENT)
	{
		ONEWIRE_WriteByte(&pxDevice->OneWireInterface, 0xCC);				/* Skip ROM 		  */
		ONEWIRE_WriteByte(&pxDevice->OneWireInterface, 0x4E);            	/* Write Scratchpad   */
		ONEWIRE_WriteByte(&pxDevice->OneWireInterface, 0xFF);            	/* TH 				  */
		ONEWIRE_WriteByte(&pxDevice->OneWireInterface, 0x00);            	/* TL 				  */
		ONEWIRE_WriteByte(&pxDevice->OneWireInterface, u8Resolution);		/* DS18B20 resolution */
		(void)ONEWIRE_Reset(&pxDevice->OneWireInterface);               	/* 1-wire reset 	  */
		ONEWIRE_WriteByte(&pxDevice->OneWireInterface, 0xCC);             	/* Skip ROM 		  */
		ONEWIRE_WriteByte(&pxDevice->OneWireInterface, 0x48);             	/* Copy Scratchpad 	  */
	}
	
	return pxDevice->Status;
}


//---------------------------------------------------------------------------------------
// Name:        DS18B20_ConvertT
// Param:		Device: pointer on the DS18B20 device structure
// Return:      1-Wire Reset result: NO_DEVICE_DETECTED, DEVICE_PRESENT or DEVICE_SHORTED
//
// Description:	Initiate a temperature conversion if the device is present and not shorted
//---------------------------------------------------------------------------------------
t_OneWireResetResult DS18B20_ConvertT(t_DS18B20Device * pxDevice)
{
	pxDevice->Status = ONEWIRE_Reset(&pxDevice->OneWireInterface);
	
	if (pxDevice->Status == DEVICE_PRESENT)
	{
		ONEWIRE_WriteByte(&pxDevice->OneWireInterface, 0xCC);            	/* Skip ROM										  */
		ONEWIRE_WriteByte(&pxDevice->OneWireInterface, 0x44);            	/* ConvertT										  */
		ONEWIRE_SetStrongPullUp(&pxDevice->OneWireInterface);          		/* Set Strong Pull-Up to power the DS18B20 device */
	}
	
	return pxDevice->Status;
}


//---------------------------------------------------------------------------------------
// Name:        DS18B20_ReadT
// Param:		Device: pointer on the DS18B20 device structure
// Return:      1-Wire Reset result: NO_DEVICE_DETECTED, DEVICE_PRESENT or DEVICE_SHORTED
//
// Description:	Read the result of the last temperature conversion (see function DS18B20_ConvertT)
//---------------------------------------------------------------------------------------
t_OneWireResetResult DS18B20_ReadT(t_DS18B20Device * pxDevice)
{
	UINT8 u8LSB, u8MSB;
		
	pxDevice->Status = ONEWIRE_Reset(&pxDevice->OneWireInterface);
	
	//To read the memory map of the DS18B20, execute a 1-wire reset 
	if (pxDevice->Status == DEVICE_PRESENT)
	{
		ONEWIRE_WriteByte(&pxDevice->OneWireInterface, 0xCC);            	/* Skip ROM								  		  */
		ONEWIRE_WriteByte(&pxDevice->OneWireInterface, 0xBE);            	/* Read DS18B20 memory map						  */
		
		u8LSB = ONEWIRE_ReadByte(&pxDevice->OneWireInterface);        		/* Temperature LSB								  */
		u8MSB = ONEWIRE_ReadByte(&pxDevice->OneWireInterface);        		/* Temperature MSB								  */
		
		(void)ONEWIRE_Reset(&pxDevice->OneWireInterface);	              	/* Reset the device to end reading the memory map */
		
		/* Temperature from the DS18B20 device  */
		pxDevice->Temperature = u8LSB + 256 * u8MSB;
	}
	
	return pxDevice->Status;
}
