/////////////////////////////////////////////////////////////////////////////////////////
//
// TheUno Ethernet Web Server Project
//
// --------------------------------------------------------------------------------------
//
// Filename:      main.c
// Version:       1.0
// Date:          15/10/2011
// Author:        Joel Guittet - http://myfreescalewebpage.free.fr
//
/////////////////////////////////////////////////////////////////////////////////////////
//
// Note about this project
//
// W5100 libraries was written with the help of various web sites, but more particularly
// with the help of projects congratulated by Circuit Cellar Wiznet Design Contest 2007,
// available at: http://www.circuitcellar.com/Wiznet. Finally, the Application Note
// AN4115 from Freescale was used for SD Card implementation.
//
/////////////////////////////////////////////////////////////////////////////////////////
//
// Revisions
//
// Version	| Author		| Description
// --------------------------------------------------------------------------------------
//			|				| 
//
/////////////////////////////////////////////////////////////////////////////////////////


//---------------------------------------------------------------------------------------
// Includes
//---------------------------------------------------------------------------------------

#include <hidef.h>
#include "derivative.h"
#include "ds18b20.h"
#include "http.h"


//---------------------------------------------------------------------------------------
// Prototypes
//---------------------------------------------------------------------------------------

static void CPU_Init(void);
static void MCG_Init(void);
static void RTC_Init(void);
static void GPIO_Init(void);


//---------------------------------------------------------------------------------------
// Global Variables
//---------------------------------------------------------------------------------------

t_DS18B20Device g_MAIN_DS18B20Device1;
t_DS18B20Device g_MAIN_DS18B20Device2;


//---------------------------------------------------------------------------------------
// Name:        main
// Param:		-
// Return:      -
//
// Description:	Main function, never return
//---------------------------------------------------------------------------------------
void main(void)
{
	/* Initialize CPU */
	CPU_Init();
	
	/* Initialize the DS18B20 sensors (10-bits resolution) */
	(void)DS18B20_Init(&g_MAIN_DS18B20Device1, &PTAD, &PTADD, 6, 10); /* First sensor is connected on A0 */
	(void)DS18B20_Init(&g_MAIN_DS18B20Device2, &PTAD, &PTADD, 5, 10); /* Second sensor is connected on A1 */
	
	/* Initialize the HTTP Server */
	HTTP_Init();
		
	EnableInterrupts;
	            
	/* Main loop */
	for(;;)
	{
		/* HTTP Server main job */
		HTTP_Server();
		
		/* Reset the watchdog */
		__RESET_WATCHDOG();
	}
}


//---------------------------------------------------------------------------------------
// Name:        CPU_Init
// Param:		-
// Return:      -
//
// Description:	Initialize CPU
//---------------------------------------------------------------------------------------
static void CPU_Init(void)
{
	/* Disable the watchdog */
	SOPT1_COPT = 0b00;
	
	/* Initialize CPU peripherals */
	MCG_Init();
	RTC_Init();
	
	/* Initialize GPIO */
	GPIO_Init();
}


//---------------------------------------------------------------------------------------
// Name:        MCG_Init
// Param:    	-
// Return:      -
//
// Description:	MCG configuration: Crystal=12MHz, Fbus=18MHz
//---------------------------------------------------------------------------------------
static void MCG_Init(void)
{
	MCGC2 = 0x36;
	while (MCGSC_OSCINIT != 1);
	MCGC1 = 0xB8;
	while (MCGSC_IREFST != 0);
	while (MCGSC_CLKST != 0b10);  
	MCGC1 = 0x98;
	MCGC3 = 0x46;
	while (MCGSC_PLLST != 1);
	while (MCGSC_LOCK != 1);
	MCGC1 = 0x18;
	while (MCGSC_CLKST != 0b11);
}


//---------------------------------------------------------------------------------------
// Name:        RTC_Init
// Param:    	-
// Return:      -
//
// Description:	Configure RTC peripheral to generates an interrupt every 500ms
//---------------------------------------------------------------------------------------
static void RTC_Init(void)
{
	RTCSC_RTCLKS = 0b00;		/* Real Time Clock source is the 1kHz low power oscillator (LP0) */
	RTCSC_RTCPS = 14;			/* Prescaler is set to divide by 500 */
	RTCMOD = 1;					/* Configure RTC Modulo Register */
	RTCSC_RTIE = 1;				/* Enable RTC Interrupts */
}


//---------------------------------------------------------------------------------------
// Name:        GPIO_Init
// Param:    	-
// Return:      -
//
// Description:	IO ports initialization
//---------------------------------------------------------------------------------------
static void GPIO_Init(void)
{
}


//---------------------------------------------------------------------------------------
// Name:        RTC_ISR
// Param:		-
// Return:      -
//
// Description:	RTC Overflow Interrupt
//				Manage DS18B20 sensors to periodically get the current temperatures 
//---------------------------------------------------------------------------------------
interrupt VectorNumber_Vrtc void RTC_ISR(void)
{
	static BOOLEAN bTemperatureReady = FALSE;
	
	RTCSC_RTIF = 1; /* Clear RTC interrupt flag */
	
	if (bTemperatureReady == FALSE)
	{
		/* Initiate a new temperature conversion on both sensors */
		/* With a resolution of 10 bits, it takes up to 188ms */
		(void)DS18B20_ConvertT(&g_MAIN_DS18B20Device1);
		(void)DS18B20_ConvertT(&g_MAIN_DS18B20Device2);
		
		/* Next step is reading the temperature */
		bTemperatureReady = TRUE;
	}
	else
	{
		/* Now read the temperature */
		/* Note that the format of the temperature is: S|2^5|2^4|2^3|2^2|2^1|2^0|2^-1|2^-2|2^-3|2^-4 */
		(void)DS18B20_ReadT(&g_MAIN_DS18B20Device1);
		(void)DS18B20_ReadT(&g_MAIN_DS18B20Device2);
		
		/* Next step is converting the temperature */
		bTemperatureReady = FALSE;
	}
}
