/////////////////////////////////////////////////////////////////////////////////////////
//
// SPI interface
//
// --------------------------------------------------------------------------------------
//
// Filename:      spi.c
// Version:       1.0
// Date:          15/10/2011
// Author:        Joel Guittet - http://myfreescalewebpage.free.fr
//
/////////////////////////////////////////////////////////////////////////////////////////
//
// Description
//
// This file is used to communicate with the SPI devices (W5100 and SD Card). It manages
// communication on the SPI interface, but is also used to get SD Card status (presence
// and write protection).
// This file can be modified to match hardware requirements, for example to modify pins
// or in order to add components on the SPI bus. 
//
/////////////////////////////////////////////////////////////////////////////////////////
//
// Revisions
//
// Version	| Author		| Description
// --------------------------------------------------------------------------------------
//			|				| 
//
/////////////////////////////////////////////////////////////////////////////////////////


//---------------------------------------------------------------------------------------
// Includes
//---------------------------------------------------------------------------------------

#include "spi.h"


//---------------------------------------------------------------------------------------
// Name:        SPI_Init_LowRate
// Param:		-
// Return:      -
//
// Description:	This function initialized the SPI interface with SPI clock set to 375kHz
//				It is used to initialized the SD Card
//---------------------------------------------------------------------------------------
void SPI_Init_LowRate(void)
{
	SPIC1 = 0x00;
	SPIC2 = 0x00;
	SPIBR = 0x52;				/* Prescaler set to 48 to get 375kHz clock (Fbus=18MHz) */
	(void)(SPIS);				/* Dummy read of the SPIS register to clear the MODF flag */
	SPIC1 = 0x50; 				/* SPI interface enabled and configured as master device */
}


//---------------------------------------------------------------------------------------
// Name:        SPI_Init_HighRate
// Param:		-
// Return:      -
//
// Description:	This function initialized the SPI interface at the maximum SPI clock
//				It is used for normal operation
//---------------------------------------------------------------------------------------
void SPI_Init_HighRate(void)
{
	SPIC1 = 0x00;
	SPIC2 = 0x00;
	SPIBR = 0x00;				/* Prescaler set to get the highest clock frequency */
	(void)(SPIS);				/* Dummy read of the SPIS register to clear the MODF flag */
	SPIC1 = 0x50; 				/* SPI interface enabled and configured as master device */
}		


//---------------------------------------------------------------------------------------
// Name:        SPI_Write
// Param:		Byte: byte to write on SPI interface
// Return:      -
//
// Description:	This function is used to write a byte on SPI interface
//---------------------------------------------------------------------------------------
void SPI_Write(UINT8 u8Byte)
{
	SPID = u8Byte;				/* Write byte */
	while (!SPIS_SPRF);			/* Wait */
	(void)(SPID);				/* Dummy read */
}


//---------------------------------------------------------------------------------------
// Name:        SPI_Read
// Param:		-
// Return:      Byte received on SPI interface
//
// Description:	This function is used to read a byte from the SPI device
//---------------------------------------------------------------------------------------
UINT8 SPI_Read(void)
{
	SPID = 0xFF;				/* To Read, you have to write */
	while (!SPIS_SPRF);			/* Wait */
	return SPID;				/* Return the byte received */ 
}


//---------------------------------------------------------------------------------------
// Name:        SPI_WriteAndRead
// Param:		Byte: byte to write on SPI interface
// Return:      Byte received on SPI interface
//
// Description:	This function is used to write a byte on SPI interface and read the one
//				sent by the SPI device
//---------------------------------------------------------------------------------------
UINT8 SPI_WriteAndRead(UINT8 u8Byte)
{
	SPID = u8Byte;				/* Write byte */
	while (!SPIS_SPRF);			/* Wait */
	return SPID;				/* Return the byte received */ 
}


//---------------------------------------------------------------------------------------
// Name:        SPI_SelectW5100Device
// Param:		-
// Return:      -
//
// Description:	This function selects the W5100 device
//---------------------------------------------------------------------------------------
void SPI_SelectW5100Device(void)
{
	PTEDD_PTEDD2 = 1;
	PTED_PTED2 = 0;
}


//---------------------------------------------------------------------------------------
// Name:        SPI_UnselectW5100Device
// Param:		-
// Return:      -
//
// Description:	This function unselects the W5100 device
//---------------------------------------------------------------------------------------
void SPI_UnselectW5100Device(void)
{
	PTEDD_PTEDD2 = 1;
	PTED_PTED2 = 1;
}


//---------------------------------------------------------------------------------------
// Name:        SPI_SelectSDCard
// Param:		-
// Return:      -
//
// Description:	This function selects the SD Card
//---------------------------------------------------------------------------------------
void SPI_SelectSDCard(void)
{
	PTADD_PTADD0 = 1;
	PTAD_PTAD0 = 0;
}


//---------------------------------------------------------------------------------------
// Name:        SPI_UnselectSDCard
// Param:		-
// Return:      -
//
// Description:	This function unselects the SD Card
//---------------------------------------------------------------------------------------
void SPI_UnselectSDCard(void)
{
	PTADD_PTADD0 = 1;
	PTAD_PTAD0 = 1;
}


//---------------------------------------------------------------------------------------
// Name:        SPI_IsSDCardPresent
// Param:		-
// Return:      TRUE if the SD Card is present, FALSE if not
//
// Description:	This function is used to know if the SD card is present
//---------------------------------------------------------------------------------------
BOOLEAN SPI_IsSDCardPresent(void)
{
	return TRUE; /* Not used on the board, return default value */
}


//---------------------------------------------------------------------------------------
// Name:        SPI_IsSDCardWriteProtected
// Param:		-
// Return:      TRUE if the SD Card is write protected, FALSE if not
//
// Description:	This function is used to know if the SD card is write protected
//---------------------------------------------------------------------------------------
BOOLEAN SPI_IsSDCardWriteProtected(void)
{
	return FALSE; /* Not used on the board, return default value */
}
